// ***************** nsk-electro.ru ***** Семерков Николай **** 2018
// OpenMonitoring_V14
// Пример кода осуществляющий чтение данных из инверторов типа SILA, Expert, Stark, Combi, Prosolar и.т.д
// и передачи для визуализации и логгирования на сервис http://open-monitoring.online

// Строки требующие редактирования в комментариях отмечены (!)

//*-- Подключение библиотек
#include <SoftwareSerial.h>
#include <string.h>
//#include <OneWire.h>                 //Раскомментировать при работе с датчиками температуры
//#include <DallasTemperature.h>       //Раскомментировать при работе с датчиками температуры


//*-- Глобальные переменные
char U_grid [6] ;        // Напряжение входной сети
char F_grid [6] ;        // Частота входной сети
char U_out [6] ;         // Выходное напряжение
int S_out ;              // Выходня мощность полная (ВА)
int P_out ;              // Выходная мощность активная (Вт)
int Load ;               // Нагрузка инвертора, (%)
short U_bat_int ;        // Напряжение АКБ - целая часть
short U_bat_fl ;         // Напряжение АКБ - дробная часть
float U_bat_dec ;        // Напряжение АКБ
int I_charge ;           // Ток заряда АКБ
int C_bat ;              // Уровень заряда АКБ (%)
int T_inv ;              // Температура инвертора (NTC)
int I_pv_bat ;           // Ток заряда АКБ от СБ
char U_pv [6] ;          // Напряжение СБ
int I_discharge ;        // Ток разряда АКБ

float Psolar = 0 ;       // Мощность снимаемая с панелей
float Power_sum = 0 ;    // Переменная, хранящая энергию до ее отправки на сервер и 
                         // получения подтверждения о ее принятии
float Power_count = 0 ;  // Количество энергии за последнюю итерацию
long delta_T = 0 ;
unsigned long preT = 0 ;

char bytes[8] = {81, 80, 73, 71, 83, 183, 169, 13};  // Строказапроса текущих параметров
long nextTime = 100000;                              // Служебная переменная

//*-- Сетевые настройки контроллера
#define SSID "XXXXX"                  // Название точки доступа     (!)
#define PASS "XXXXXXXX"               // Пароль                     (!)
String IP = "open-monitoring.online"; // Адрес сервера (не менять)

//*-- Параметры полученные при регистрации на http://open-monitoring.online
String ID = "XXX";                    // ID                         (!)
String KEY = "XXXXXX";                // Код доступа                (!)
long interval = 65000;                // Периодичность отправки пакетов на сервер

#define esp_bitrate 9600              // Скорость передачи данных между arduino и ESP8266 
                                      // (0 -> RX, 1 -> TX)
#define com_bitrate 2400              // Скорость передачи данных между arduino и инвертором
#define com_rxpin 12                  // RX
#define com_txpin 13                  // TX

/*-- Пины аналоговых каналов (при наличии)
#define one_wire_pin 2                // Вход для подключения датчиков тем-ры 
#define AnalogIn1 A0                  // Аналоговый вход 1  (Напряжение АКБ1) 
#define AnalogIn2 A1                  // Аналоговый вход 1  (Напряжение АКБ2)



//*-- Работа с несколькими датчиками по одной шине
OneWire oneWire(one_wire_pin);
DallasTemperature sensors(&oneWire);
*/

//*-- Software Serial
SoftwareSerial comSerial( com_rxpin, com_txpin ); // RX, TX

//*-- GET Information
String GET = "GET /get?cid=" + ID + "&key=" + KEY; // GET request
String HOST = "Host: " + IP + "\r\n\r\n";

void setup() {

  Serial.begin(esp_bitrate);
  Serial.setTimeout(2000);      // Лимит времени ожидания порта 2 сек.
  comSerial.begin(com_bitrate); //Инициализация порта для обмена с инвертором
  //sensors.begin();            //Инициализация опроса датчиков температуры DS18B20 (если используются)

  Serial.println("AT");
  delay(2000);
  if (Serial.find("OK"))
  {
  //  Serial.println("ESP: OK\nReady to sent!");    // Для отладки
    connectWiFi();
  }
  
  //*-- Тестовый опрос инвертора --*//
  comSerial.println("Q");
  delay(2000);

  while (comSerial.available()) {comSerial.read();} // Очищаем порт
  //*--Конец тестового опроса инвертора--*//
}


void loop() {
  /*-- Измерение температуры                               // Если используется
  sensors.requestTemperatures();
  int t1 = round(sensors.getTempCByIndex(0));
  delay(2000);
  int t2 = round(sensors.getTempCByIndex(1));
  delay(2000);

  String T1 = String(t1); // turn integer to string
  String T2 = String(t2); // turn integer to string

  //*-- Измерение напряжения делителем
  float Volt1 = 0;
  float Volt2 = 0;
  for (int i = 0; i < 100; i++) {
    Volt1 = Volt1 + (0.0495 * analogRead(AnalogIn1)) / 100; // Пропорция 26,14/2,57В
    Volt2 = Volt2 + (0.0495 * analogRead(AnalogIn2)) / 100; // Пропорция 26,14/2,57В
    delay(1);
  }
  String Uakb2 = String(Volt2);
  String Uakb1 = String(Volt1 - Volt2);
  */


  //*-- Запрос текущих параметров инвертора --*//
  //Serial.println("Inv request"); //*Для отладки
  comSerial.print(bytes);
  delay(2000);

  int i = 0;
  char buffer[120];

  // Если есть данные - читаем
  if (comSerial.available()) {
    delay(100);

    //Загоняем прочитанное в буфер
    while ( comSerial.available() && i < 119) {
      buffer[i++] = comSerial.read();
    }
    buffer[i++] = '\0';  //закрываем массив
  }

  //если буфер наполнен
  if (i > 0) {
    //Serial.println(buffer); //*Для отладки
    //Разбираем ответ
    sscanf(buffer, "(%s%s%s%*s%d%d%d%*d%d%*c%d%d%d%d%d%s%*s%d", &U_grid, &F_grid, &U_out, &S_out, &P_out, &Load, &U_bat_int, &U_bat_fl, &I_charge, &C_bat, &T_inv, &I_pv_bat, &U_pv, &I_discharge );
  }
  // U_bat в десятичном виде
  U_bat_dec = U_bat_int + .01*U_bat_fl ;


  //*-- Расчет энергии --*//                    
  Psolar = U_bat_dec * I_pv_bat ;                // Текущая мощность
  delta_T = millis() - preT ;                    // Время предыдущей итерации
  if ( abs(delta_T) > 604800000 ) {              // Проверка на переполнение millis
    delta_T = 0;
  }
  Power_count = Psolar * delta_T * .0000002778 ; // Количество энергии за 1 итерацию
  Power_sum += Power_count ;
  preT = millis() ;

  //Serial.print("\n");
  //Serial.println(millis());
  //Serial.print("Энергия.: ");
  //Serial.println(Power_sum);

 
  // Проверяем вышел ли тайм-аут сервера и если да, то отправляем данные
  if ( millis() - nextTime > (interval - 4215) ) {

  // Переводим целые в строку
  String Sout = String(S_out);
  String Pout = String(P_out);
  String L = String(Load);
  String U_bat = String(U_bat_dec);
  String Icharge = String(I_charge);
  String Cbat = String(C_bat);
  String Ipvbat = String(I_pv_bat);
  String Idischarge = String(I_discharge);
  String Power = String(Power_sum);

    update_param( U_grid, F_grid, U_out, Sout, Pout, L, U_bat, Icharge, Cbat, Ipvbat, U_pv, Idischarge, Power); // Отправляем пакет на сервер
    nextTime = millis();
  }
}


//*-- Функции --*//
//*-- Соединение с точкой доступа
boolean connectWiFi() {
  Serial.println("AT+CWMODE=1");    // Режим работы "клиент"
  delay(2000);
  String cmd = "AT+CWJAP=\"";       // Подключение к точке доступа
  cmd += SSID;
  cmd += "\",\"";
  cmd += PASS;
  cmd += "\"";
  Serial.println(cmd);
  delay(5000);
  if (Serial.find("OK"))
  {
    // Serial.println("Connect: OK\n");  // Для отладки
    return true;
  }
  else
  {
    //Serial.println("Connect: Error");  // Для отладки
    return false;
  }

  cmd = "AT+CIPMUX=0";                   // Режим одиночного соединения
  Serial.println( cmd );
  if ( Serial.find( "Error") )
  {
   // Serial.print( "RECEIVED: Error" ); // Для отладки
    return false;
  }
}


//*--- Отправка данных на сервер
void update_param( String U_grid, String F_grid, String U_out, String Sout, String Pout, String L, String U_bat, String Icharge, String Cbat, String Ipvbat, String U_pv, String Idischarge, String Power ) {

  String cmd = "AT+CIPSTART=\"TCP\",\"";// Setup TCP connection
  cmd += IP;
  cmd += "\",80";
  Serial.println(cmd);
  delay(1000);

  if ( Serial.find( "Error" ) )
  {
    //Serial.print( "RECEIVED: Error\nExit1" ); // Для отладки
    return;
  }

  // Отправляем собранную строку на сервер
  cmd = GET + "&p1=" + U_grid + "&p2=" + F_grid + "&p3=" + U_out + "&p4=" + Sout + "&p5=" + Pout + "&p6=" + L + "&p7=" + U_bat + "&p8=" + Icharge + "&p9=" + Cbat + "&p10=" + Ipvbat + "&p11=" + U_pv + "&p12=" + Idischarge ;
  cmd += "&p13=" + Power;
  cmd += " HTTP/1.1\r\n"  + HOST;
  Serial.print( "AT+CIPSEND=" );
  Serial.println( cmd.length() );
  if (Serial.find( ">" ) )
  {
    Serial.print(cmd);
    delay(30);
  }
  else
  {
    Serial.println( "AT+CIPCLOSE" );//close TCP connection
  }
  // Устаревший вариант сброса накопленной энергии
  /*if ( Serial.find("OK") )
  {
    //Serial.println( "RECEIVED: OK" );           // Для отладки
    Power_sum = 0 ;                               // Обнуляем накопленную энергию
  }
  else
  {
    //Serial.println( "RECEIVED: Error\nExit2" ); // Для отладки
  }*/
  while (Serial.available()) {
    String line = Serial.readStringUntil('\r');
    if (line == "\n1<br>") {
      Serial.println("Сбрасываем Power_sum");
      Power_sum = 0 ; // Обнуляем накопленную энергию
    }
    if (line.length() < 16 ) {
      Serial.print(line);
    }
  }
}

// Знаешь как сделать лучше? Напиши мне на почту: semerkov@mail.ru!
//Спасибо!
